@@ -1,5 +1,4 @@
 import re
-import subprocess
 import unicodedata
 import pyjf3
 import kaa
@@ -10,65 +9,202 @@ from gappedbuf import re as gre
 
 
 class CursorCommands(Commands):
+    
     @command('cursor.right')
     @norerun
     def right(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.right()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.right.select')
+    @norerun
+    def right_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.right()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.left')
     @norerun
     def left(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.left()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.left.select')
+    @norerun
+    def left_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.left()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.up')
     @norerun
     def up(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.up()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.up.select')
+    @norerun
+    def up_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.up()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.down')
     @norerun
     def down(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.down()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.down.select')
+    @norerun
+    def down_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.down()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.prev-line')
+    @norerun
+    def prev_line(self, wnd):
+        wnd.screen.selection.end_cursor()
+        wnd.cursor.prev_line()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.next-line')
+    @norerun
+    def next_line(self, wnd):
+        wnd.screen.selection.end_cursor()
+        wnd.cursor.next_line()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.word-right')
     @norerun
     def word_right(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.right(word=True)
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.word-right.select')
+    @norerun
+    def word_right_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.right(word=True)
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.word-left')
     @norerun
     def word_left(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.left(word=True)
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.word-left.select')
+    @norerun
+    def word_left_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.left(word=True)
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.pagedown')
     @norerun
     def pagedown(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.pagedown()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.pagedown.select')
+    @norerun
+    def pagedown_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.pagedown()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.pageup')
     @norerun
     def pageup(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.pageup()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.pageup.select')
+    @norerun
+    def pageup_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.pageup()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.home')
     @norerun
     def home(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.home()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.home.select')
+    @norerun
+    def home_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.home()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.end')
     @norerun
     def end(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.end()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.end.select')
+    @norerun
+    def end_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.end()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.top-of-line')
+    @norerun
+    def tol(self, wnd):
+        wnd.screen.selection.end_cursor()
+        wnd.cursor.tol(wnd.cursor.pos)
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.end-of-line')
+    @norerun
+    def eol(self, wnd):
+        wnd.screen.selection.end_cursor()
+        wnd.cursor.eol(wnd.cursor.pos)
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.top-of-file')
     @norerun
     def top(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.tof()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.top-of-file.select')
+    @norerun
+    def top_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.tof()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.end-of-file')
     @norerun
     def last(self, wnd):
+        wnd.screen.selection.end_cursor()
         wnd.cursor.eof()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
+
+    @command('cursor.end-of-file.select')
+    @norerun
+    def last_select(self, wnd):
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.cursor.eof()
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @command('cursor.go-to-line')
     @norerun
@@ -86,6 +222,8 @@ class CursorCommands(Commands):
                 kaa.app.messagebar.set_message('Enter valid line number.')
                 return
 
+            wnd.screen.selection.end_cursor()
+            
             pos = wnd.document.get_lineno_pos(lineno)
             tol = wnd.document.gettol(pos)
             wnd.cursor.setpos(wnd.cursor.adjust_nextpos(wnd.cursor.pos, tol))
@@ -93,7 +231,6 @@ class CursorCommands(Commands):
             popup = w.get_label('popup')
             popup.destroy()
 
-
         def filter(wnd, s):
             return re.match(r'\d*', s).group()
 
@@ -105,29 +242,34 @@ class CursorCommands(Commands):
 
 
 class ScreenCommands(Commands):
-    @command('screen.selection.begin')
+    @command('selection.begin-cursor')
     @norerun
     def selection_begin(self, wnd):
-        wnd.screen.selection.start_selection(wnd.cursor.pos)
+        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
 
-    @command('screen.selection.set-end')
+    @command('selection.set-to')
     @norerun
     def selection_set_end(self, wnd):
-        wnd.screen.selection.set_end(wnd.cursor.pos)
+        wnd.screen.selection.set_to(wnd.cursor.pos)
 
-    @command('screen.selection.clear')
+    @command('selection.end-cursor')
     @norerun
     def selection_clear(self, wnd):
-        wnd.screen.selection.clear()
+        wnd.screen.selection.end_cursor()
 
-    @command('screen.selection.all')
+    @command('selection.set-mark')
+    @norerun
+    def selection_set_mark(self, wnd):
+        wnd.screen.selection.set_mark(wnd.cursor.pos)
+
+    @command('selection.all')
     @norerun
     def select_all(self, wnd):
         f = wnd.cursor.adjust_nextpos(wnd.cursor.pos, 0)
         t = wnd.cursor.adjust_nextpos(wnd.cursor.pos, wnd.document.endpos())
         wnd.screen.selection.set_range(f, t)
 
-    @command('screen.selection.curline')
+    @command('selection.curline')
     @norerun
     def select_cur_line(self, wnd):
         tol = wnd.cursor.adjust_nextpos(
@@ -139,7 +281,7 @@ class ScreenCommands(Commands):
 
         wnd.screen.selection.set_range(tol, eol)
 
-    @command('screen.selection.curword')
+    @command('selection.curword')
     @norerun
     def select_cur_word(self, wnd):
         f, t = wnd.document.mode.get_word_at(wnd.cursor.pos)
@@ -149,7 +291,7 @@ class ScreenCommands(Commands):
 
         wnd.screen.selection.set_range(f, t)
 
-    @command('screen.selection.expand_sel')
+    @command('selection.expand-sel')
     @norerun
     @norec
     def expand_sel(self, wnd):
@@ -176,12 +318,12 @@ class ScreenCommands(Commands):
     @norerun
     def lineselection_begin(self, wnd):
         tol = wnd.cursor.adjust_nextpos(wnd.document.gettol(wnd.cursor.pos))
-        wnd.screen.selection.start_selection(tol)
+        wnd.screen.selection.begin_cursor(tol)
 
     @command('screen.lineselection.set-end')
     @norerun
     def lineselection_set_end(self, wnd):
-        f = wnd.screen.selection.start
+        f = wnd.screen.get_start()
         if f is None:
             self.select_cur_line(wnd)
         else:
@@ -300,9 +442,6 @@ class EditCommands(Commands):
 
     @command('edit.delete.line')
     def delete_line(self, wnd):
-        if self.delete_sel(wnd):
-            return
-
         pos = wnd.cursor.pos
         nextpos = wnd.cursor.adjust_nextpos(pos, wnd.document.find_newline(pos))
         if pos < nextpos:
@@ -310,9 +449,6 @@ class EditCommands(Commands):
 
     @command('edit.delete.currentline')
     def delete_currentline(self, wnd):
-        if self.delete_sel(wnd):
-            return
-
         pos = wnd.cursor.pos
         f = wnd.cursor.adjust_nextpos(pos, wnd.document.gettol(pos))
         t = wnd.cursor.adjust_nextpos(pos, wnd.document.geteol(f))
@@ -360,24 +496,6 @@ class EditCommands(Commands):
         wnd.screen.selection.clear()
         indent = wnd.document.mode.on_auto_indent(wnd)
 
-    def get_line_sel(self, wnd):
-        doc = wnd.document
-        sel = wnd.screen.selection.get_range()
-        if not sel:
-            f = t = wnd.cursor.pos
-        else:
-            f, t = sel
-
-        tol =  doc.gettol(f)
-
-        t_tol =  doc.gettol(t)
-        if t != t_tol:
-            eol = doc.geteol(t)
-        else:
-            eol = t
-
-        return tol, eol
-
     def _indent_line(self, wnd, pos):
         mode = wnd.document.mode
         f, t = mode.get_indent_range(pos)
@@ -406,7 +524,7 @@ class EditCommands(Commands):
         wnd.document.undo.beginblock()
         try:
             mode = wnd.document.mode
-            while tol < wnd.screen.selection.end:
+            while tol < wnd.screen.selection.get_end():
                 f, t = mode.get_indent_range(tol)
                 if f != t:
                     cols = mode.calc_cols(f, t)
@@ -418,8 +536,8 @@ class EditCommands(Commands):
                 tol = doc.geteol(tol)
         finally:
             wnd.document.undo.endblock()
-
-        wnd.cursor.setpos(wnd.screen.selection.start)
+        f, t = wnd.screen.selection.get_range()
+        wnd.cursor.setpos(f)
         wnd.cursor.savecol()
 
     def _dedent_line(self, wnd, pos):
@@ -446,7 +564,7 @@ class EditCommands(Commands):
         wnd.document.undo.beginblock()
         try:
             mode = wnd.document.mode
-            while tol < wnd.screen.selection.end:
+            while tol < wnd.screen.selection.get_end():
                 f, t = mode.get_indent_range(tol)
                 if f != t:
                     cols = mode.calc_cols(f, t)
@@ -460,7 +578,8 @@ class EditCommands(Commands):
                 tol = doc.geteol(tol)
         finally:
             wnd.document.undo.endblock()
-        wnd.cursor.setpos(wnd.screen.selection.start)
+        f, t = wnd.screen.selection.get_range()
+        wnd.cursor.setpos(f)
         wnd.cursor.savecol()
 
     def _undo(self, wnd, rec):
@@ -525,16 +644,20 @@ class EditCommands(Commands):
 
             self.on_edited(wnd)
 
-    @command('edit.copy')
-    def copy(self, wnd):
+    def _copy_sel(self, wnd):
         sel = wnd.screen.selection.get_range()
         if sel:
             f, t = sel
             kaa.app.clipboard = wnd.document.gettext(f, t)
+        
+    @command('edit.copy')
+    def copy(self, wnd):
+        self._copy_sel(wnd)
+        wnd.screen.selection.set_mark(None)
 
     @command('edit.cut')
     def cut(self, wnd):
-        self.copy(wnd)
+        self._copy_sel(wnd)
         self.delete_sel(wnd)
         wnd.screen.selection.clear()
 
@@ -579,38 +702,6 @@ class EditCommands(Commands):
             self.put_string(wnd, text)
             wnd.screen.selection.clear()
 
-    @command('edit.paste-lines')
-    @norerun
-    def edit_pastelines(self, wnd):
-        from kaa.ui.pastelines import pastelinesmode
-        doc = pastelinesmode.PasteLinesMode.build(wnd)
-
-        kaa.app.show_dialog(doc)
-
-    @command('tools.execute-shell-command')
-    @norerun
-    def execute_shell_command(self, wnd):
-        def callback(w, s):
-            s = s.strip()
-            if s:
-                ret = subprocess.check_output(
-                    s, stderr=subprocess.STDOUT,
-                    shell=True,
-                    universal_newlines=True)
-
-                wnd.document.mode.edit_commands.put_string(wnd, ret)
-                kaa.app.messagebar.set_message(
-                    "{} letters inserted".format(len(ret)))
-
-                popup = w.get_label('popup')
-                popup.destroy()
-
-        from kaa.ui.inputline import inputlinemode
-        doc = inputlinemode.InputlineMode.build('Shell command:', callback)
-        kaa.app.messagebar.set_message("Execute shell command")
-
-        kaa.app.show_dialog(doc)
-
 
 class CodeCommands(Commands):
     @command('code.region.linecomment')
@@ -630,15 +721,16 @@ class CodeCommands(Commands):
             wnd.document.undo.beginblock()
             try:
                 mode = wnd.document.mode
-                while tol < wnd.screen.selection.end:
+                while tol < wnd.screen.selection.get_end():
                     wnd.document.mode.edit_commands.insert_string(
                         wnd, tol, wnd.document.mode.LINE_COMMENT, 
                         update_cursor=False)
                     tol = wnd.document.geteol(tol)
             finally:
                 wnd.document.undo.endblock()
-    
-            wnd.cursor.setpos(wnd.screen.selection.start)
+
+            f, t = wnd.screen.selection.get_range()
+            wnd.cursor.setpos(f)
             wnd.cursor.savecol()
 
     def _is_comment_line(self, wnd, pos):
@@ -663,7 +755,7 @@ class CodeCommands(Commands):
             wnd.document.undo.beginblock()
             try:
                 mode = wnd.document.mode
-                while tol < wnd.screen.selection.end:
+                while tol < wnd.screen.selection.get_end():
                     m = self._is_comment_line(wnd, tol)
                     if m:
                         f, t = m.span(1)
@@ -672,8 +764,9 @@ class CodeCommands(Commands):
                     tol = wnd.document.geteol(tol)
             finally:
                 wnd.document.undo.endblock()
-    
-            wnd.cursor.setpos(wnd.screen.selection.start)
+
+            f, t = wnd.screen.selection.get_range()
+            wnd.cursor.setpos(f)
             wnd.cursor.savecol()
 
 class MacroCommands(Commands):
@@ -746,8 +839,7 @@ class SearchCommands(Commands):
         if hit:
             f, t = hit.span()
             wnd.cursor.setpos(f)
-            wnd.screen.selection.start = f
-            wnd.screen.selection.end = t
+            wnd.screen.selection.set_range(f, t)
             kaa.app.messagebar.set_message('found')
         else:
             kaa.app.messagebar.set_message('not found')
@@ -810,17 +902,3 @@ class SearchCommands(Commands):
         kaa.app.show_dialog(doc)
 
 
-class RerunCommand(Commands):
-    @command('command.rerun')
-    @norerun
-    def reruncommand(self, wnd):
-        mode = wnd.document.mode
-        for commandid in kaa.app.lastcommands:
-            is_available, command = mode.get_command(commandid)
-            if not command:
-                msg = 'command {!r} is not registered.'.format(commandid)
-                kaa.app.messagebar.set_message(msg)
-                kaa.log.error(msg)
-                return
-            command(wnd)
-
