@@ -50,7 +50,7 @@ class ModeBase:
     SCREEN_BUILD_ENTIRE_ROWS = False
     SHOW_LINENO = False
     USE_UNDO = True
-    DOCUMENT = False
+    DOCUMENT_MODE = False
     HIGHLIGHT_CURSORLINE = False
 
     tab_width = 8
@@ -262,7 +262,8 @@ class ModeBase:
             self.current_lastcommands = None
 
     def on_esc_pressed(self, wnd, event):
-        pass
+        # cancel mark
+        wnd.screen.selection.set_mark(None)
 
     def on_cursor_located(self, wnd, pos, y, x):
         pass
