@@ -173,70 +173,114 @@ class Selection:
         self._marks = document.Marks()
         self.screen = screen
 
-    @property
-    def start(self):
+    def _get_cursor_start(self):
         return self._marks.get('start', None)
 
-    @start.setter
-    def start(self, pos):
+    def _set_cursor_start(self, pos):
         self._marks['start'] = pos
-
-    @property
-    def end(self):
-        return self._marks.get('end', None)
-
-    @end.setter
-    def end(self, pos):
-        self._marks['end'] = pos
-
-    def clear(self):
-        """Clear selection"""
-        changed = self.is_started()
-        self.start = self.end = None
-
-        if changed:
-            self.screen.style_updated()
-
-    def is_started(self):
+        
+    def _is_cursor_started(self):
         """Return True if range was selected"""
 
-        return self.start is not None
+        return self._get_cursor_start() is not None
     
+    def get_end(self):
+        return self._marks.get('end', None)
+
+    def _set_end(self, pos):
+        self._marks['end'] = pos
+
+    def _set_mark(self, pos):
+        self._marks['mark'] = pos
+        
+    def _get_mark(self):
+        return self._marks.get('mark', None)
+        
+    def _has_mark(self):
+        return self._get_mark() is not None
+        
     def is_selected(self):
-        if self.is_started():
+        if self._is_cursor_started() or self._has_mark():
             sel = self.get_range()
             if sel:
                 f, t, = sel
                 return f != t
-                
-    def start_selection(self, pos):
-        """Start range selection if it was not started"""
 
-        if not self.is_started():
-            self.start = self.end = pos
+    def set_mark(self, pos):
+        cur = self.get_range()
 
-    def set_end(self, pos):
-        """Update where selection ends."""
-
-        if not self.is_started():
+        self._set_mark(pos)
+        if pos is not None:
+            self._set_cursor_start(None)
+        if cur != self.get_range():
+            self.screen.style_updated()
+            
+    def begin_cursor(self, pos):
+        """Start cursor range selection if it was not started"""
+        
+        if self._has_mark():
             return
+        
+        if self._is_cursor_started():
+            return
+            
+        self._set_cursor_start(pos)
+        self._set_end(pos)
 
-        changed = self.end != pos
-        self.end = pos
+    def set_to(self, pos):
+        """Update where selection to."""
+
+        if not self._has_mark() and not self._is_cursor_started():
+            return
+        changed = self.get_end() != pos
+        self._set_end(pos)
+
+        if changed:
+            self.screen.style_updated()
+        
+    def end_cursor(self):
+        """Clear selection"""
+
+        changed = self._is_cursor_started()
+        
+        self._set_cursor_start(None)
+        self._set_end(None)
+
+        if changed:
+            self.screen.style_updated()
+
+    def clear(self):
+        """Clear selection"""
+
+        changed = self._is_cursor_started() is not None
+        if not changed:
+            changed =  self._get_mark() is not None
+            
+        self._set_cursor_start(None)
+        self._set_end(None)
+        self._set_mark(None)
 
         if changed:
             self.screen.style_updated()
 
     def get_range(self):
-        if self.start is None or self.end is None or (self.start == self.end):
+        start = self._get_cursor_start()
+        if start is None:
+            start = self._get_mark()
+            
+        end = self.get_end()
+        if start is None or end is None or (start == end):
             return None
 
-        return tuple(sorted((self.start, self.end)))
+        return tuple(sorted((start, end)))
 
     def set_range(self, f, t):
-        if (self.start, self.end) != (f, t):
-            self.start = f
-            self.end = t
+        start = self._get_cursor_start()
+        end = self.get_end()
+        if (start, end) != (f, t):
+            self._set_cursor_start(f)
+            self._set_end(t)
+            self._set_mark(None)
             self.screen.style_updated()
 
     def on_document_updated(self, pos, inslen, dellen):
@@ -448,8 +492,7 @@ class Screen:
 
         p = 0
         col = col - row.wrapindent
-        for pos, charcols, c, i in zip(row.positions, row.cols,
-                                       row.chars, row.intervals):
+        for pos, charcols, c in zip(row.positions, row.cols, row.chars):
             if (p + charcols > col) or (c == '\n'):
                 return pos
             p += charcols
@@ -504,6 +547,24 @@ class Screen:
 
         return cols
 
+    def get_pos_at_cols(self, tol, cols):
+        posidx, row = self.getrow(tol)
+        if not row:
+            return tol
+
+        p = 0
+        for n in range(posidx, len(self.rows)):
+            row = self.rows[n]
+            if row.tol != tol:
+                break
+
+            for pos, charcols, c in zip(row.positions, row.cols, row.chars):
+                if p + charcols > cols:
+                    return pos
+                p += charcols
+
+        return self.document.find_newline(tol)
+        
     def _fillscreen(self):
         while True:
             bottomrow = self.rows[-1]
